-module(n4u_sup).
-behaviour(supervisor).

-include_lib("n4u/include/n4u.hrl").

-export([start_link/0, init/1]).
-export([proc/2]).


ets_tables() -> [ cookies, actions, globals, caching ].
ets_tables_opts() -> [ set, named_table, { keypos, 1 }, public ].


start_link() ->
  A = supervisor:start_link({local, ?MODULE}, ?MODULE, []),
  n4u_async:start(#handler{module=?MODULE, class=system, group=?MODULE, state=[], name="timer"}),
  A.


init([]) ->
  [ ets:new(T, ets_tables_opts()) || T <- ets_tables() ],
  
  case application:get_env(n4u, mq, "") of
    n4u_syn -> syn:init();
    _ -> ok
  end,
  
  Procs = [],
  {ok, {{one_for_one, 10, 10}, Procs}}.



% proc handler for n4u_async:start (line 16 of this module)
timer_restart(Diff) ->
  {X, Y, Z} = Diff,
  erlang:send_after(1000 * (Z + 60 * Y + 60 * 60 * X), erlang:self(), {timer, ping}).


ping() -> application:get_env(n4u, timer, {0, 10, 0}).


proc(init, #handler{}=Async) ->
  wf:info(?MODULE, "Proc Init: ~p~n", [init]),
  Timer = timer_restart(ping()),
  {ok, Async#handler{state=Timer}};

proc({timer, ping}, #handler{state=Timer}=Async) ->
  case Timer of
    undefined -> skip;
    _ -> erlang:cancel_timer(Timer)
  end,
  wf:info(?MODULE, "n4u Timer: ~p~n", [ping]),
  (application:get_env(n4u, session, n2o_session)):invalidate_sessions(),
  wf:invalidate_cache(),
  {reply, ok, Async#handler{state=timer_restart(ping())}}.
